@@ -189,6 +189,15 @@ static void bts_model_rbs2k_e1line_bind_ops(struct e1inp_line *line)
 
 static int bts_model_rbs2k_bts_init(struct gsm_bts *bts)
 {
+	if (bts->nr > 0xff) {
+		/* Currently unsupported, since bts->nr is used directly filling
+		 * in uint8_t abis_om2000 OML fields. See abis_om2k_bts_init().
+		 */
+		LOGP(DNM, LOGL_ERROR, "bts %u type %s > 255 is unsupported! "
+		     "Please configure your bts type %s first in the bts list\n",
+		     bts->nr, bts->model->name, bts->model->name);
+		return -ENOTSUP;
+	}
 	abis_om2k_bts_init(bts);
 	return 0;
 }
@@ -212,6 +221,14 @@ static struct gsm_bts_model model_rbs2k = {
 };
 
 static int bts_model_rbs2k_start(struct gsm_network *net)
+{
+	osmo_signal_register_handler(SS_L_INPUT, inp_sig_cb, NULL);
+	osmo_signal_register_handler(SS_L_GLOBAL, gbl_sig_cb, NULL);
+
+	return 0;
+}
+
+int bts_model_rbs2k_init(void)
 {
 	model_rbs2k.features.data = &model_rbs2k._features_data[0];
 	model_rbs2k.features.data_len = sizeof(model_rbs2k._features_data);
@@ -222,13 +239,5 @@ static int bts_model_rbs2k_start(struct gsm_network *net)
 	osmo_bts_set_feature(&model_rbs2k.features, BTS_FEAT_HSCSD);
 	osmo_bts_set_feature(&model_rbs2k.features, BTS_FEAT_MULTI_TSC);
 
-	osmo_signal_register_handler(SS_L_INPUT, inp_sig_cb, NULL);
-	osmo_signal_register_handler(SS_L_GLOBAL, gbl_sig_cb, NULL);
-
-	return 0;
-}
-
-int bts_model_rbs2k_init(void)
-{
 	return gsm_bts_model_register(&model_rbs2k);
 }
