@@ -49,8 +49,8 @@ struct gsm_bts_trx_ts *ts_fi_ts(struct osmo_fsm_inst *fi)
 
 static void ts_fsm_update_id(struct gsm_bts_trx_ts *ts)
 {
-	osmo_fsm_inst_update_id_f(ts->fi, "%u-%u-%u-%s", ts->trx->bts->nr, ts->trx->nr, ts->nr,
-				  gsm_pchan_id(ts->pchan_on_init));
+	osmo_fsm_inst_update_id_f_sanitize(ts->fi, '_', "%u-%u-%u-%s", ts->trx->bts->nr, ts->trx->nr, ts->nr,
+					   gsm_pchan_name(ts->pchan_on_init));
 }
 
 static __attribute__((constructor)) void ts_fsm_init(void)
@@ -323,11 +323,37 @@ static void ts_fsm_not_initialized_onenter(struct osmo_fsm_inst *fi, uint32_t pr
 	chan_counts_ts_clear(ts);
 }
 
-static void ts_fsm_unused_onenter(struct osmo_fsm_inst *fi, uint32_t prev_state)
+static void ts_fsm_unused_pdch_act(struct osmo_fsm_inst *fi)
 {
 	struct gsm_bts_trx_ts *ts = ts_fi_ts(fi);
 	struct gsm_bts *bts = ts->trx->bts;
 
+	if (bts->gprs.mode == BTS_GPRS_NONE) {
+		LOG_TS(ts, LOGL_DEBUG, "GPRS mode is 'none': not activating PDCH.\n");
+		return;
+	}
+
+	if (!ts->pdch_act_allowed) {
+		LOG_TS(ts, LOGL_DEBUG, "PDCH is disabled for this timeslot,"
+		       " either due to a PDCH ACT NACK, or from manual VTY command:"
+		       " not activating PDCH. (last error: %s)\n",
+		       ts->last_errmsg ? : "-");
+		return;
+	}
+
+	if (bsc_co_located_pcu(bts) && !pcu_connected(bts->network)) {
+		LOG_TS(ts, LOGL_DEBUG, "PCU not connected: not activating PDCH.\n");
+		return;
+	}
+
+	osmo_fsm_inst_state_chg(fi, TS_ST_WAIT_PDCH_ACT, CHAN_ACT_DEACT_TIMEOUT,
+				T_CHAN_ACT_DEACT);
+}
+
+static void ts_fsm_unused_onenter(struct osmo_fsm_inst *fi, uint32_t prev_state)
+{
+	struct gsm_bts_trx_ts *ts = ts_fi_ts(fi);
+
 	chan_counts_ts_update(ts);
 
 	/* We are entering the unused state. There must by definition not be any lchans waiting to be
@@ -342,19 +368,7 @@ static void ts_fsm_unused_onenter(struct osmo_fsm_inst *fi, uint32_t prev_state)
 	switch (ts->pchan_on_init) {
 	case GSM_PCHAN_OSMO_DYN:
 	case GSM_PCHAN_TCH_F_PDCH:
-		if (bts->gprs.mode == BTS_GPRS_NONE) {
-			LOG_TS(ts, LOGL_DEBUG, "GPRS mode is 'none': not activating PDCH.\n");
-			return;
-		}
-		if (!ts->pdch_act_allowed) {
-			LOG_TS(ts, LOGL_DEBUG, "PDCH is disabled for this timeslot,"
-			       " either due to a PDCH ACT NACK, or from manual VTY command:"
-			       " not activating PDCH. (last error: %s)\n",
-			       ts->last_errmsg ? : "-");
-			return;
-		}
-		osmo_fsm_inst_state_chg(fi, TS_ST_WAIT_PDCH_ACT, CHAN_ACT_DEACT_TIMEOUT,
-					T_CHAN_ACT_DEACT);
+		ts_fsm_unused_pdch_act(fi);
 		break;
 
 	case GSM_PCHAN_CCCH_SDCCH4_CBCH:
@@ -399,6 +413,10 @@ static void ts_fsm_unused(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 		/* ignored. */
 		return;
 
+	case TS_EV_PDCH_ACT:
+		ts_fsm_unused_pdch_act(fi);
+		return;
+
 	default:
 		OSMO_ASSERT(false);
 	}
@@ -522,6 +540,10 @@ static void ts_fsm_pdch(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 			}
 		}
 
+	case TS_EV_PDCH_DEACT:
+		ts_fsm_pdch_deact(fi);
+		return;
+
 	case TS_EV_LCHAN_UNUSED:
 		/* ignored */
 		return;
@@ -880,6 +902,7 @@ static const struct osmo_fsm_state ts_fsm_states[] = {
 		.in_event_mask = 0
 			| S(TS_EV_LCHAN_REQUESTED)
 			| S(TS_EV_LCHAN_UNUSED)
+			| S(TS_EV_PDCH_ACT)
 			,
 		.out_state_mask = 0
 			| S(TS_ST_WAIT_PDCH_ACT)
@@ -912,6 +935,7 @@ static const struct osmo_fsm_state ts_fsm_states[] = {
 		.in_event_mask = 0
 			| S(TS_EV_LCHAN_REQUESTED)
 			| S(TS_EV_LCHAN_UNUSED)
+			| S(TS_EV_PDCH_DEACT)
 			,
 		.out_state_mask = 0
 			| S(TS_ST_WAIT_PDCH_DEACT)
@@ -983,6 +1007,8 @@ static const struct value_string ts_fsm_event_names[] = {
 	OSMO_VALUE_STRING(TS_EV_PDCH_ACT_NACK),
 	OSMO_VALUE_STRING(TS_EV_PDCH_DEACT_ACK),
 	OSMO_VALUE_STRING(TS_EV_PDCH_DEACT_NACK),
+	OSMO_VALUE_STRING(TS_EV_PDCH_DEACT),
+	OSMO_VALUE_STRING(TS_EV_PDCH_ACT),
 	{}
 };
 
@@ -1096,3 +1122,23 @@ bool ts_usable_as_pchan(struct gsm_bts_trx_ts *ts, enum gsm_phys_chan_config as_
 
 	return true;
 }
+
+void ts_pdch_act(struct gsm_bts_trx_ts *ts)
+{
+	/* We send the activation event only when the timeslot is UNUSED. In all other cases the timeslot FSM
+	 * will automatically handle the activation at some later point. */
+	if (ts->fi->state != TS_ST_UNUSED)
+		return;
+
+	osmo_fsm_inst_dispatch(ts->fi, TS_EV_PDCH_ACT, NULL);
+}
+
+void ts_pdch_deact(struct gsm_bts_trx_ts *ts)
+{
+	/* We send the deactivation event only when the timeslot is active as PDCH. The timeslot FSM will check
+	 * if the PCU is available before activating the PDCH again. */
+	if (ts->fi->state != TS_ST_PDCH)
+		return;
+
+	osmo_fsm_inst_dispatch(ts->fi, TS_EV_PDCH_DEACT, NULL);
+}
